import argparse

from update_toml.models.user_input import UserInput


class CLIInputHandler:
    def parse(self) -> UserInput:
        parser = argparse.ArgumentParser(
            prog="UpdateTOML",
            description="Update the value in a TOML file from a CLI",
        )

        parser.add_argument(
            "-f",
            "--file",
            help="The path to the .toml file to update",
            default="pyproject.toml",
        )
        parser.add_argument(
            "-p",
            "--path",
            help="Path in the attribute in the TOML file to update (ex. project.version or tool.poetry.version)",
            required=True,
            type=str,
        )
        parser.add_argument(
            "-v",
            "--value",
            help="The value to set the path to in the TOML file",
            required=True,
            type=str,
        )

        args: argparse.Namespace = parser.parse_args()

        return UserInput(args.file, args.path, args.value)



from __future__ import annotations

import json
from typing import Any

import tomlkit as toml

from update_toml.exceptions.file_not_loaded_exception import FileNotLoadedException


class TOMLFile:
    def __init__(self, file_path: str = "pyproject.toml") -> None:
        self._file_path: str = file_path
        self._contents: dict | None = None

    def load(self) -> None:
        with open(self._file_path, encoding="utf-8") as f:
            self._contents = toml.load(f)

    def to_json(self) -> str:
        if self._contents is None:
            raise FileNotLoadedException("load has not yet been called")

        return json.dumps(self._contents)

    def update(self, path: str, new_value: str) -> None:
        if self._contents is None:
            raise FileNotLoadedException("load has not yet been called")

        path_parts: list[str] = path.split(".")

        if len(path_parts) < 2:
            raise ValueError(
                "Path should have at least two parts (ex. project.version)"
            )

        parent_object: dict = self._get_parent_object(path_parts, self._contents)
        property_to_update: str = path_parts[0]
        parent_object[property_to_update] = new_value

    def get_value(self, path: str) -> str:
        if self._contents is None:
            raise FileNotLoadedException("load has not yet been called")

        path_parts: list[str] = path.split(".")
        return self._get_value(path_parts, self._contents)

    def save(self) -> None:
        if not self._contents:
            raise ValueError(
                "TOML file has not yet been loaded. Please call load() first."
            )

        with open(file=self._file_path, mode="w", encoding="utf-8") as f:
            toml.dump(self._contents, f)

    def _get_parent_object(self, path_parts: list[str], current_object: Any) -> dict:
        if len(path_parts) > 1:
            current_path: str = path_parts.pop(0)
            return self._get_parent_object(path_parts, current_object[current_path])

        return current_object

    def _get_value(self, path_parts: list[str], current_object: dict) -> Any:
        if len(path_parts) > 1:
            current_path: str = path_parts.pop(0)
            return self._get_value(path_parts, current_object[current_path])

        return current_object[path_parts[0]]



#!/usr/bin/env

import logging
import os

from update_toml.cli_input_handler import CLIInputHandler
from update_toml.interfaces.input_handler import InputHandler
from update_toml.models.user_input import UserInput
from update_toml.toml_file import TOMLFile

logging.basicConfig(
    format="%(levelname)s: %(message)s",
    level=os.environ.get("LOGLEVEL", "INFO"),
)


def main() -> None:
    input_handler: InputHandler = CLIInputHandler()
    args: UserInput = input_handler.parse()

    toml_file = TOMLFile(args.toml_path)

    toml_file.load()
    logging.info(f"Loaded {args.toml_path}.")

    toml_file.update(args.path, args.value)
    logging.info(f"Updated {args.path} to '{args.value}'.")

    toml_file.save()
    logging.info(f"Saved update values to {args.toml_path}.")


if __name__ == "__main__":
    main()
